#!/usr/bin/env perl
#
# External script which computes and returns the MP2 energy
# computed from MO integrals in the matrix element file.
#

use strict;
use MatEl;
use OpMat '&lind2';
use GauComplex;

my ($me) = MatEl->new(1);
#
# Read the file taking the name from the sixth argument if present
# for use as as script for the Gaussian External interface;
# otherwise use the first argument.
if (@ARGV >= 6) {$me->read($ARGV[5]);}
else  {$me->read($ARGV[0]);}
my ($aoer) = $me->item("alpha orbital energies");
my ($aoe) = $aoer->array();
my ($boer) = $me->item("beta orbital energies");
my ($boe) = $boer->array();
my ($aa2er) = $me->item("aa mo 2e integrals");
my ($aa2e) = $aa2er->array();
my ($ab2er) = $me->item("ab mo 2e integrals");
my ($ab2e);
if (ref($ab2er)) {$ab2e = $ab2er->array();}
my ($ba2er) = $me->item("ba mo 2e integrals");
my ($ba2e) = $ba2er->array();
my ($bb2er) = $me->item("bb mo 2e integrals");
my ($bb2e);
if (ref($ab2er)) {$bb2e = $bb2er->array();}
my ($ne,$multip) = ($me->NE,$me->MULTIP);
my ($nae,$nbe) = (($ne+$multip-1)/2,($ne-$multip+1)/2);
my ($nbu,$nfc,$nfv) = ($me->NBSUSE,$me->NFC,$me->NFV);
my ($e2aa,$e2ab,$e2bb,$t2aa,$t2ab,$t2bb) = (0.0e0,0.0e0,0.0e0);
my ($noa,$nob,$nva,$nvb) = ($nae-$nfc,$nbe-$nfc,$nbu-$nae-$nfv,$nbu-$nbe-$nfv);
if ($me->itran != 4) {die "can only process partial set of transformed integrals";}
print "nov $noa $nob $nva $nvb nfc $nfc itran $me->itran\n";

($e2aa,$t2aa) = domp2_xx ($nfc,$noa,$nva,$aoe,$aa2e);
($e2ab,$t2ab) = domp2_xy ($nfc,$noa,$nob,$nva,$nvb,$aoe,$boe,$ba2e);
($e2bb,$t2bb) = domp2_xx ($nfc,$nob,$nvb,$boe,$bb2e);

my ($e2) = $e2aa + $e2ab + $e2bb;
print "ehf=",$me->ESCF,"\n";
print "t2aa=",$t2aa," e2aa=",$e2aa,"\n";
print "t2ab=",$t2ab," e2ab=",$e2ab,"\n";
print "t2bb=",$t2bb," e2bb=",$e2bb,"\n";
my ($eump2) = $me->ESCF + $e2;
print "e2=$e2 eump2=$eump2\n";

my ($men) = MatEl->new(1);
$men->copy_header($me);

$men->EUMP2($eump2);
$men->ETOTAL($eump2);

# update the file if running as external
if (@ARGV >= 6) {$men->write($ARGV[5]);}

sub domp2_xx {
  my ($nfc,$nox,$nvx,$xoe,$xx2e) = @_;
  my ($i,$j,$a,$b,$ei,$eij,$eija,$eijab,$indi,$aa,$iajb,$ibja,$db,$t);
  my ($e2,$t2) = (0.0e0,0.0e0);
  my ($nr) = $nox + $nvx;
  my ($nrtt) = ($nr*($nr+1))/2;
  for ($i = 2; $i <= $nox; $i++) {
    $ei = $xoe->[$i-1+$nfc];
    $indi = $nrtt*$nr*($i-1);
    for ($j = 1; $j < $i; $j++) {
      $eij = $ei + $xoe->[$j-1+$nfc];
      for ($a = (2+$nox); $a <= $nr; $a++) {
	$eija = $xoe->[$a-1+$nfc] - $eij;
	$aa = ($a*($a-1))/2;
	for ($b = (1+$nox); $b < $a; $b++) {
	  $eijab = $eija + $xoe->[$b-1+$nfc];
	  $iajb = $indi + $nrtt*($a-1) + lind2($j,$b);
	  $ibja = $indi + $nrtt*($b-1) + lind2($j,$a);
	  $db = $xx2e->[$iajb] - $xx2e->[$ibja];
	  $t = $db / $eijab;
	  $e2 = $e2 - $t*$db;
	  $t2 = $t2 + $t*$t;
	}
      }
    }
  }
  return ($e2,$t2);
}
sub domp2_xy {
  my ($nfc,$noa,$nob,$nva,$nvb,$aoe,$boe,$ba2e) = @_;
  my ($i,$j,$a,$b,$ei,$eij,$eija,$eijab,$indi,$ja,$iajb,$db,$t);
  my ($e2,$t2) = (0.0e0,0.0e0);
  my ($nr) = $noa + $nva;
  my ($nrtt) = ($nr*($nr+1))/2;
  for ($i = 1; $i <= $noa; $i++) {
    $ei = $aoe->[$i-1+$nfc];
    $indi = $nrtt*$nr*($i-1);
    for ($j = 1; $j <= $nob; $j++) {
      $eij = $ei + $boe->[$j-1+$nfc];
      for ($a = (1+$noa); $a <= $nr; $a++) {
	$eija = $aoe->[$a-1+$nfc] - $eij;
	for ($b = (1+$nob); $b <= $nr; $b++) {
	  $eijab = $eija + $boe->[$b-1+$nfc];
	  $iajb = $indi + $nrtt*($a-1) + lind2($b,$j);
	  $db = $ba2e->[$iajb];
	  $t = $db / $eijab;
	  $e2 = $e2 - $t*$db;
	  $t2 = $t2 + $t*$t;
	}
      }
    }
  }
  return ($e2,$t2);
}
